import config from '@/config'
import { computed, type Ref, watch } from 'vue'
import { getTile, tileToWorldXY } from '@/services/zone'
import { useGameStore } from '@/stores/game'
import { useZoneEditorStore } from '@/stores/zoneEditor'

export function usePointerHandlers(scene: Phaser.Scene, layer: Phaser.Tilemaps.TilemapLayer, waypoint: Ref<{ visible: boolean; x: number; y: number }>, camera: Ref<Phaser.Cameras.Scene2D.Camera>, isDragging: Ref<boolean>) {
  const gameStore = useGameStore()
  const zoneEditorStore = useZoneEditorStore()

  function updateWaypoint(pointer: Phaser.Input.Pointer) {
    const { x: px, y: py } = camera.value.getWorldPoint(pointer.x, pointer.y)
    const pointerTile = getTile(px, py, layer)

    if (!pointerTile) {
      waypoint.value.visible = false
      return
    }

    const worldPoint = tileToWorldXY(layer, pointerTile.x, pointerTile.y)
    waypoint.value = {
      visible: true,
      x: worldPoint.position_x,
      y: worldPoint.position_y + config.tile_size.y + 15
    }
  }

  const isMoveTool = computed(() => zoneEditorStore.tool === 'move')

  watch(isMoveTool, (newValue) => {
    if (newValue) {
      scene.input.on(Phaser.Input.Events.POINTER_MOVE, dragZone)
    } else {
      scene.input.off(Phaser.Input.Events.POINTER_MOVE, dragZone)
    }
  })

  function dragZone(pointer: Phaser.Input.Pointer) {
    if (!isDragging.value) return
    const { x, y, prevPosition } = pointer
    camera.value.scrollX -= (x - prevPosition.x) / camera.value.zoom
    camera.value.scrollY -= (y - prevPosition.y) / camera.value.zoom
  }

  function clickTile(pointer: Phaser.Input.Pointer) {
    const { x: px, y: py } = camera.value.getWorldPoint(pointer.x, pointer.y)
    const pointerTile = getTile(px, py, layer)

    if (!pointerTile) {
      return
    }
    gameStore.connection?.emit('character:move', { position_x: pointerTile.x, position_y: pointerTile.y })
  }

  function handleZoom(pointer: Phaser.Input.Pointer, _: unknown, __: unknown, deltaY: number) {
    if (pointer.event instanceof WheelEvent && pointer.event.altKey) {
      scene.scale.setZoom(scene.scale.zoom - deltaY * 0.01)
      camera.value = scene.cameras.main
    }
  }

  function setupPointerHandlers() {
    scene.input.on(Phaser.Input.Events.POINTER_MOVE, updateWaypoint)
    scene.input.on(Phaser.Input.Events.POINTER_WHEEL, handleZoom)
    scene.input.on(Phaser.Input.Events.POINTER_MOVE, dragZone)

    // These are for in-game only, not for in the zone editor
    if (!zoneEditorStore.active) {
      scene.input.on(Phaser.Input.Events.POINTER_UP, clickTile)
    }
  }

  function cleanupPointerHandlers() {
    scene.input.off(Phaser.Input.Events.POINTER_MOVE, updateWaypoint)
    scene.input.off(Phaser.Input.Events.POINTER_WHEEL, handleZoom)
    scene.input.off(Phaser.Input.Events.POINTER_MOVE, dragZone)

    // These are for in-game only, not for in the zone editor
    if (!zoneEditorStore.active) {
      scene.input.off(Phaser.Input.Events.POINTER_UP, clickTile)
    }
  }

  return {
    setupPointerHandlers,
    cleanupPointerHandlers
  }
}
